
-- return the window size
getWindowSize = getWindowSize or function()
	return 907, 415
end

-- create and lay out all the controls
createUI = createUI or function()

	createUICommon()

	-- constants
	local kKnobSpace = 45
	local kKnobSpaceV = 70
	local kWaveformW = 433
	local kWaveformH = 151
	local kDropdownW = 72
	local kTextW = 40
	local kLabelSpaceV = 30
	local kRandomW = 433
	local kRandomH = 18

	local function knobGroup( LR, id, title )
		g = group( title )
		
		local x = 0
		local y = 0

		theKnobRenderer = smallKnobRenderer

		smallKnobAndTextField( g, "Speed " .. LR,		"Speed",			x, y )
		smallKnobAndTextField( g, "Beats " .. LR,		"Beats",			x, y+kKnobSpaceV )
		x = x + kKnobSpace

		labelledDropDown( g, "Speed Multiplier " .. LR, "Multiplier", x, y, kDropdownW )
		y = y + kLabelSpaceV
		labelledDropDown( g, "Speed Mode " .. LR, "Speed Mode", x, y, kDropdownW )
		y = y + kLabelSpaceV
		labelledDropDown( g, "Sync " .. LR, "Sync", x, y, kDropdownW )
		y = y + kLabelSpaceV
		labelledDropDown( g, "Beat Divisor " .. LR, "Divisor", x, y, kDropdownW )
		y = y + kTextFieldH + 4
		g:add( paramButton( "Triplet " .. LR, "Triplet", x, y, kDropdownW, kTextFieldH ) )
		
		x = x + kDropdownW + 10
		y = 0

		labelledDropDown( g, "Swing Type " .. LR, "Swing Type", x, y, kDropdownW )
		y = y + kLabelSpaceV
		labelledDropDown( g, "Interpolation " .. LR, "Interpolation", x, y, kDropdownW )
		y = y + kLabelSpaceV
		labelledDropDown( g, "Range Mode " .. LR, "Range", x, y, kDropdownW )
		y = y + kLabelSpaceV
		labelledDropDown( g, "Length Mode " .. LR, "Length Mode", x, y, kDropdownW )
		y = y + kLabelSpaceV

		x = x + kDropdownW + 10
		y = 0

		local x1 = x
		
		smallKnobAndTextField( g, "Swing " .. LR,			"Swing",	x, y )
		x = x + kKnobSpace
		smallKnobAndTextField( g, "Gate Length " .. LR,		"Gate",		x, y, kTextW )
		x = x + kKnobSpace
		smallKnobAndTextField( g, "Phase " .. LR,			"Phase",	x, y, kTextW )
		x = x + kKnobSpace
		smallKnobAndTextField( g, "Asymmetry " .. LR,		"Asym",		x, y, kTextW )
		x = x + kKnobSpace
		smallKnobAndTextField( g, "Smoothing " .. LR,		"Smooth",	x, y )
		x = x + kKnobSpace

		local x2 = x

		x = x1
		y = y + kKnobSpaceV

		smallKnobAndTextField( g, "Offset " .. LR,			"Offset",	x, y, kTextW )
		x = x + kKnobSpace
		smallKnobAndTextField( g, "Multiplier " .. LR,		"Scale",	x, y, kTextW )
		x = x + kKnobSpace
		smallKnobAndTextField( g, "Start Step " .. LR,			"Start",	x, y )
		x = x + kKnobSpace
		smallKnobAndTextField( g, "Length In Steps " .. LR,		"Length",	x, y )
		x = x + kKnobSpace
		smallKnobAndTextField( g, "End Step " .. LR,			"End",	x, y )
		x = x + kKnobSpace

		return g
	end

	local function makeResetGroup( LR, id, title )
		g = group( title )
		
		local x = 0
		local y = 0

		theKnobRenderer = smallKnobRenderer

		labelledDropDown( g, "Signal Input " .. LR, "Signal", x, y+1.5*kLabelSpaceV, kKnobSize )
		labelledDropDown( g, "Input Mode " .. LR, "Input Mode", x+kKnobSpace, y+1.5*kLabelSpaceV, kDropdownW )
		labelledDropDown( g, "Trigger Input " .. LR, "Trigger", x, y+0*kLabelSpaceV, kKnobSize )
		labelledDropDown( g, "Reset Input " .. LR, "Reset", x, y+3*kLabelSpaceV, kKnobSize )
		labelledDropDown( g, "Reset By Note " .. LR, "Reset Mode", x+kKnobSpace, y+3*kLabelSpaceV, kDropdownW )
		x = x + kKnobSpace + kDropdownW + 2*pad
		smallKnobAndTextField( g, "Reset Note Min " .. LR,		"Reset Min",		x, y + kKnobSpaceV, nil, "%.0f" )
		smallKnobAndTextField( g, "Trigger Note Min " .. LR,		"Trigger Min",		x, y, nil, "%.0f" )
		x = x + 1.5*kKnobSpace
		smallKnobAndTextField( g, "Reset Note Max " .. LR,		"Reset Max",		x, y + kKnobSpaceV, nil, "%.0f" )
		smallKnobAndTextField( g, "Trigger Note Max " .. LR,		"Trigger Max",		x, y, nil, "%.0f" )
		x = x + kKnobSpace

		return g
	end

	local function makeRandomGroup( LR, id, title )
		g = group( title )
		
		local x = 0
		local y = 0
		local y2 = y + kKnobSpaceV

		theKnobRenderer = smallKnobRenderer
		
		local kButtonH = 20

		labelledDropDown( g, "Random Mode " .. LR, "Mode", x, y, kDropdownW )
		y = y + 30
		g:add( paramButton( "Random Set Next " .. LR, "Set Next", x, y, kDropdownW, kButtonH ) )
		y = y + 30
		g:add( paramButton( "Random Enable " .. LR, "Enable", x, y, kDropdownW, kButtonH ) )
		y = 0
		x = x + kDropdownW + pad
		smallKnobAndTextField( g, "Register Length " .. LR,		"Length",		x, y2, kTextW )
		smallKnobAndTextField( g, "DAC Bits " .. LR,		"DAC Bits",		x, y, kTextW )
		x = x + 1.5*kKnobSpace
		smallKnobAndTextField( g, "Randomness " .. LR,		"Randomness",		x, y, kTextW )
		smallKnobAndTextField( g, "DAC Scale " .. LR,		"Scale",			x, y2, kTextW )
		x = x + 1.5*kKnobSpace

		local kDACOptionW = 68
		local kDACEditorH = 112
		y = y-kTextFieldH-4
		local dacEd = dacEditor( id, x, y, kDACOptionW*3+10, kDACEditorH )
		setDACEditorColours( dacEd )
		g:add( dacEd )
		y = y + kDACEditorH + 5
		g:add( paramButton( "DAC Enable " .. LR, "Enable", x, y, kDACOptionW, kTextFieldH ) )
		x = x + kDACOptionW + 5
		g:add( dropDown( "DAC Scale Mode " .. LR, x, y, kDACOptionW, kTextFieldH ) )
		x = x + kDACOptionW + 5
		g:add( dropDown( "DAC Range Mode " .. LR, x, y, kDACOptionW, kTextFieldH ) )

		return g
	end

	-- make groups and knobs
	--

	theKnobRenderer = smallKnobRenderer

	miscGroup = group( "OSC" )
	smallKnobAndTextField( miscGroup, "OSC Port Offset",	"Port",		0*kKnobSpace, 0 )
	miscGroup:setVisible( false )

	local mainGroup = {}
	mainGroup.L = knobGroup( "L", 0, "Left Channel" )
	mainGroup.R = knobGroup( "R", 1, "Right Channel" )
	
	local resetGroup = {}
	resetGroup.L = makeResetGroup( "L", 0, "Left Channel Inputs" )
	resetGroup.R = makeResetGroup( "R", 1, "Right Channel Inputs" )
	resetGroup.L:setVisible( false )
	resetGroup.R:setVisible( false )

	local randomGroup = {}
	randomGroup.L = makeRandomGroup( "L", 0, "Left Channel Random" )
	randomGroup.R = makeRandomGroup( "R", 1, "Right Channel Random" )
	randomGroup.L:setVisible( false )
	randomGroup.R:setVisible( false )

	-- positions!

	local xbase = windowEdgePad
	local ybase = windowEdgePad
	local x = xbase
	local y = ybase
	
	mainGroup.R:setPosition( x, y, true )
	resetGroup.R:setPosition( x, y, true )
	randomGroup.R:setPosition( x, y, true )
	x = x + mainGroup.R:width() + pad
	local stepEdX = x
	
	miscGroup:setPosition( x, y, true )
	x = x + miscGroup:width() + pad
	
	x = xbase
	y = y + mainGroup.R:height() + pad
	local randomRightY = y
	y = y + kRandomH + pad
	local ybase2 = y
	
	mainGroup.L:setPosition( x, y, true )
	resetGroup.L:setPosition( x, y, true )
	randomGroup.L:setPosition( x, y, true )
	x = x + mainGroup.L:width() + pad
	
	y = y + mainGroup.L:height() + pad
	local randomLeftY = y

	-- add directly positioned objects
	--

	local stepEds = {}
	stepEds.R = stepEditor( 1, stepEdX, ybase, kWaveformW, kWaveformH )
	setStepEditorColours( stepEds.R )
	add( stepEds.R )
	stepEds.L = stepEditor( 0, stepEdX, ybase2, kWaveformW, kWaveformH )
	setStepEditorColours( stepEds.L )
	add( stepEds.L )
	
	local bigEdW = stepEdX+kWaveformW-xbase
	local bigEds = {}
	bigEds.R = stepEditor( 1, xbase, ybase, bigEdW, kWaveformH )
	setStepEditorColours( bigEds.R )
	add( bigEds.R )
	elementSetVisible( bigEds.R, false )
	bigEds.L = stepEditor( 0, xbase, ybase2, bigEdW, kWaveformH )
	setStepEditorColours( bigEds.L )
	add( bigEds.L )
	elementSetVisible( bigEds.L, false )

	local function buttonRow( LR, id, x, y )
		local x, y = x, y
		local kButtonW = 60
		add( luaButton( function() button1( mainGroup[LR], resetGroup[LR], randomGroup[LR], stepEds[LR], bigEds[LR] ) end, "Main", "Show main controls", "push", x, y, kButtonW, kRandomH ) )
		x = x + kButtonW + pad
		add( luaButton( function() button2( mainGroup[LR], resetGroup[LR], randomGroup[LR], stepEds[LR], bigEds[LR] ) end, "Inputs", "Show input/reset/trigger controls", "push", x, y, kButtonW, kRandomH ) )
		x = x + kButtonW + pad
		add( luaButton( function() button3( mainGroup[LR], resetGroup[LR], randomGroup[LR], stepEds[LR], bigEds[LR] ) end, "Random", "Show random random controls", "push", x, y, kButtonW, kRandomH ) )
		x = x + kButtonW + pad
		add( luaButton( function() button4( mainGroup[LR], resetGroup[LR], randomGroup[LR], stepEds[LR], bigEds[LR] ) end, "Big", "Show big step editor", "push", x, y, kButtonW, kRandomH ) )
		x = x + kButtonW + pad
		local reg = randomRegister( id, stepEdX, y, kRandomW, kRandomH )
		setRandomRegisterColours( reg )
		add( reg )
	end
	
	buttonRow( "L", 0, xbase, randomLeftY )
	buttonRow( "R", 1, xbase, randomRightY )

end		-- of createUI()

button1 = function( g, rg, rndg, s, b )
	g:setVisible( true )
	rg:setVisible( false )
	rndg:setVisible( false )
	elementSetVisible( s, true )
	elementSetVisible( b, false )
	redrawLabels()
end

button2 = function( g, rg, rndg, s, b )
	g:setVisible( false )
	rg:setVisible( true )
	rndg:setVisible( false )
	elementSetVisible( s, true )
	elementSetVisible( b, false )
	redrawLabels()
end

button3 = function( g, rg, rndg, s, b )
	g:setVisible( false )
	rg:setVisible( false )
	rndg:setVisible( true )
	elementSetVisible( s, true )
	elementSetVisible( b, false )
	redrawLabels()
end

button4 = function( g, rg, rndg, s, b )
	g:setVisible( false )
	rg:setVisible( false )
	rndg:setVisible( false )
	elementSetVisible( s, false )
	elementSetVisible( b, true )
	redrawLabels()
end
